@@ -79,14 +79,22 @@ class Widget:
         try:
             extra = field.get_value('extra')
         except KeyError:
-    # In case extra is not defined as in DateTimeWidget
+            # In case extra is not defined as in DateTimeWidget
             extra = ''
         return render_element("input",
                               type="hidden",
                               name=key,
                               value=value,
                               extra=extra)
-                              
+
+    def render_view(self, field, value):
+        """Renders this widget for public viewing.
+        """
+        # default implementation
+        if value is None:
+            return ''
+        return value
+
 class TextWidget(Widget):
     """Text widget
     """
@@ -140,6 +148,11 @@ class TextWidget(Widget):
                                   size=field.get_value('display_width'),
                                   extra=field.get_value('extra'))
 
+    def render_view(self, field, value):
+        if value is None:
+            return ''
+        return value
+    
 TextWidgetInstance = TextWidget()
 
 class PasswordWidget(TextWidget):
@@ -166,6 +179,9 @@ class PasswordWidget(TextWidget):
                                   size=field.get_value('display_width'),
                                   extra=field.get_value('extra'))
 
+    def render_view(self, field, value):
+        return "[password]"
+    
 PasswordWidgetInstance = PasswordWidget()
 
 class CheckBoxWidget(Widget):
@@ -195,6 +211,12 @@ class CheckBoxWidget(Widget):
                                   css_class=field.get_value('css_class'),
                                   extra=field.get_value('extra'))
 
+    def render_view(self, field, value):
+        if value:
+            return 1
+        else:
+            return 0
+        
 CheckBoxWidgetInstance = CheckBoxWidget()
 
 class TextAreaWidget(Widget):
@@ -228,7 +250,7 @@ class TextAreaWidget(Widget):
     def render(self, field, key, value, REQUEST):
         width = field.get_value('width')
         height = field.get_value('height')
-            
+        
         return render_element("textarea",
                               name=key,
                               css_class=field.get_value('css_class'),
@@ -236,10 +258,18 @@ class TextAreaWidget(Widget):
                               rows=height,
                               contents=html_quote(value),
                               extra=field.get_value('extra'))
-            
+
+    def render_view(self, field, value):
+        if value is None:
+            return ''
+        return value
+    
 TextAreaWidgetInstance = TextAreaWidget()
 
 class LinesTextAreaWidget(TextAreaWidget):
+    property_names = Widget.property_names +\
+                     ['width', 'height', 'view_separator', 'extra']
+    
     default = fields.LinesField('default',
                                 title='Default',
                                 description=(
@@ -247,10 +277,26 @@ class LinesTextAreaWidget(TextAreaWidget):
                                 default=[],
                                 width=20, height=3,
                                 required=0)
+
+    view_separator = fields.StringField('view_separator',
+                                        title='View separator',
+                                        description=(
+        "When called with render_view, this separator will be used to "
+        "render individual items."),
+                                        width=20,
+                                        default='<br />\n',
+                                        whitespace_preserve=1,
+                                        required=1)
+    
     def render(self, field, key, value, REQUEST):
         value = string.join(value, "\n")
         return TextAreaWidget.render(self, field, key, value, REQUEST)
 
+    def render_view(self, field, value):
+        if value is None:
+            return ''
+        return string.join(value, field.get_value('view_separator'))
+        
 LinesTextAreaWidgetInstance = LinesTextAreaWidget()
 
 class FileWidget(TextWidget):
@@ -276,7 +322,10 @@ class FileWidget(TextWidget):
                                   value=value,
                                   size=field.get_value('display_width'),
                                   extra=field.get_value('extra'))
-        
+
+    def render_view(self, field, value):
+        return "[File]"
+    
 FileWidgetInstance = FileWidget()
 
 class ItemsWidget(Widget):
@@ -324,6 +373,7 @@ class SingleItemsWidget(ItemsWidget):
                                       default=0)    
 
     def render_items(self, field, key, value, REQUEST):
+            
         # get items
         items = field.get_value('items')
     
@@ -347,6 +397,7 @@ class SingleItemsWidget(ItemsWidget):
                 item_text = item
                 item_value = item
 
+                
             if item_value == value and not selected_found:
                 rendered_item = self.render_selected_item(item_text,
                                                           item_value,
@@ -376,11 +427,22 @@ class MultiItemsWidget(ItemsWidget):
                                 width=20, height=3,
                                 default=[],
                                 required=0)
-        
+
+    view_separator = fields.StringField('view_separator',
+                                        title='View separator',
+                                        description=(
+        "When called with render_view, this separator will be used to "
+        "render individual items."),
+                                        width=20,
+                                        default='<br />\n',
+                                        whitespace_preserve=1,
+                                        required=1)
+    
     def render_items(self, field, key, value, REQUEST):
         # need to deal with single item selects
         if type(value) is not type([]):
             value = [value]
+
         items = field.get_value('items')
         css_class = field.get_value('css_class')
         rendered_items = []
@@ -406,6 +468,30 @@ class MultiItemsWidget(ItemsWidget):
 
         return rendered_items
 
+    def render_items_view(self, field, value):
+        if type(value) is not type([]):
+            value = [value]
+
+        items = field.get_value('items')
+        d = {}
+        for item in items:
+            try:
+                item_text, item_value = item
+            except ValueError:
+                item_text = item
+                item_value = item
+            d[item_value] = item_text
+        result = []
+        for e in value:
+            result.append(d[e])
+        return result
+
+    def render_view(self, field, value):
+        if value is None:
+            return ''
+        return string.join(self.render_items_view(field, value),
+                           field.get_value('view_separator'))
+    
 class ListWidget(SingleItemsWidget):
     """List widget.
     """
@@ -423,7 +509,7 @@ class ListWidget(SingleItemsWidget):
 
     def render(self, field, key, value, REQUEST):
         rendered_items = self.render_items(field, key, value, REQUEST)
-
+    
         return render_element('select',
                               name=key,
                               css_class=field.get_value('css_class'),
@@ -437,6 +523,11 @@ class ListWidget(SingleItemsWidget):
     def render_selected_item(self, text, value, key, css_class):
         return render_element('option', contents=text, value=value,
                               selected=None)
+
+    def render_view(self, field, value):
+        if value is None:
+            return ''
+        return value
     
 ListWidgetInstance = ListWidget()
 
@@ -444,7 +535,7 @@ class MultiListWidget(MultiItemsWidget):
     """List widget with multiple select.
     """
     property_names = Widget.property_names +\
-                     ['items', 'size', 'extra']
+                     ['items', 'size', 'view_separator', 'extra']
     
     size = fields.IntegerField('size',
                                title='Size',
@@ -514,14 +605,19 @@ class RadioWidget(SingleItemsWidget):
                               name=key,
                               value=value,
                               checked=None) + text
-       
+
+    def render_view(self, field, value):
+        if value is None:
+            return ''
+        return value
+        
 RadioWidgetInstance = RadioWidget()
 
 class MultiCheckBoxWidget(MultiItemsWidget):
     """multiple checkbox widget.
     """
     property_names = Widget.property_names +\
-                     ['items', 'orientation']
+                     ['items', 'orientation', 'view_separator']
     
     orientation = fields.ListField('orientation',
                                    title='Orientation',
@@ -556,19 +652,19 @@ class MultiCheckBoxWidget(MultiItemsWidget):
                               name=key,
                               value=value,
                               checked=None) + text
-       
+
+    
 MultiCheckBoxWidgetInstance = MultiCheckBoxWidget()
 
 class DateTimeWidget(Widget):
     property_names = Widget.property_names +\
                      ['default_now', 'date_separator', 'time_separator',
                       'input_style', 'input_order',
-                      'date_only']
+                      'date_only','ampm_time_style']
 
     default = fields.DateTimeField('default',
                                    title="Default",
-                                   description=(
-        "The default datetime."),
+                                   description=("The default datetime."),
                                    default=None,
                                    display_style="text",
                                    display_order="ymd",
@@ -630,13 +726,22 @@ class DateTimeWidget(Widget):
         "Display the date only, not the time."),
                                      default=0)
 
+    ampm_time_style = fields.CheckBoxField('ampm_time_style',
+                                           title="AM/PM time style",
+                                           description=(
+        "Display time in am/pm format."),
+                                           default=0)
+
+
     # FIXME: do we want to handle 'extra'?
     
     def render(self, field, key, value, REQUEST):
+        use_ampm = field.get_value('ampm_time_style')
         # FIXME: backwards compatibility hack:
         if not hasattr(field, 'sub_form'):
             from StandardFields import create_datetime_text_sub_form
             field.sub_form = create_datetime_text_sub_form()
+            
         if value is None and field.get_value('default_now'):
             value = DateTime()
         if value is None:
@@ -645,12 +750,17 @@ class DateTimeWidget(Widget):
             day = None
             hour = None
             minute = None
+            ampm = None
         else:
             year = "%04d" % value.year()
             month = "%02d" % value.month()
             day = "%02d" % value.day()
-            hour = "%02d" % value.hour()
+            if use_ampm:
+                hour = "%02d" % value.h_12()
+            else:
+                hour = "%02d" % value.hour()
             minute = "%02d" % value.minute()
+            ampm = value.ampm()
         
         input_order = field.get_value('input_order')
         if input_order == 'ymd':
@@ -674,10 +784,47 @@ class DateTimeWidget(Widget):
             time_result = (field.render_sub_field('hour', hour, REQUEST) +
                            field.get_value('time_separator') +
                            field.render_sub_field('minute', minute, REQUEST))
+            
+            if use_ampm:
+                time_result += '&nbsp;' + field.render_sub_field('ampm', ampm, REQUEST)
+                
             return date_result + '&nbsp;&nbsp;&nbsp;' + time_result
         else:
             return date_result
-                       
+
+    def render_view(self, field, value):
+        if value is None:
+            return ''
+
+        use_ampm = field.get_value('ampm_time_style')
+        
+        year = "%04d" % value.year()
+        month = "%02d" % value.month()
+        day = "%02d" % value.day()
+        if use_ampm:
+            hour = "%02d" % value.h_12()
+        else:
+            hour = "%02d" % value.hour()
+        minute = "%02d" % value.minute()
+        ampm = value.ampm()
+        
+        order = field.get_value('input_order')
+        if order == 'ymd':
+            output = [year, month, day]
+        elif order == 'dmy':
+            output = [day, month, year]
+        elif order == 'mdy':
+            output = [month, day, year]
+        date_result = string.join(output, field.get_value('date_separator'))
+        
+        if not field.get_value('date_only'):
+            time_result = hour + field.get_value('time_separator') + minute
+            if use_ampm:
+                time_result += '&nbsp;' + ampm                
+            return date_result + '&nbsp;&nbsp;&nbsp;' + time_result
+        else:
+            return date_result
+        
 DateTimeWidgetInstance = DateTimeWidget()
 
 def render_tag(tag, **kw):
@@ -702,7 +849,7 @@ def render_tag(tag, **kw):
     for key, value in kw.items():
         if value == None:
             value = key
-        attr_list.append('%s="%s"' % (key, html_quote(str(value))))
+        attr_list.append('%s="%s"' % (key, html_quote(value)))
             
     attr_str = string.join(attr_list, " ")
     return "<%s %s %s" % (tag, attr_str, extra)
